/*
 * libpri: An implementation of Primary Rate ISDN
 *
 * Written by Mark Spencer <markster@linux-support.net>
 *
 * Copyright (C) 2001, Linux Support Services, Inc.
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */
 
#ifndef _PRI_INTERNAL_H
#define _PRI_INTERNAL_H

#include <sys/time.h>

struct pri_sched {
	struct timeval when;
	void (*callback)(void *data);
	void (*callback2)(void *data, char);
	void *data;
	char hasdata2;
	char data2;
};

struct q921_frame;
enum q931_state;
enum q931_mode;

/* No more than 128 scheduled events */
#define MAX_SCHED 128

#define Q921_MAX_TEIS 8
#define Q921_TEI_BASE 64

struct pri {
	int fd;				/* File descriptor for D-Channel */
	struct pri_sched pri_sched[MAX_SCHED];	/* Scheduled events */
	int debug;			/* Debug stuff */
	int state;			/* State of D-channel */
	int switchtype;		/* Switch type */
	int localtype;		/* Local network type (unknown, network, cpe) */
	int remotetype;		/* Remote network type (unknown, network, cpe) */
	
	/* Q.921 State */
 	int q921_state[Q921_MAX_TEIS];	
 	char dchanup;
 	
 	/* TEI registry */
     	char q921_teis[Q921_MAX_TEIS];
  	
 	char tei;
 	unsigned int ri; 
 
 	int cref;			/* Next call reference value */  // let's think about this for a while...
  	
 	int busy[Q921_MAX_TEIS];			/* Peer is busy */
  
 	int window[Q921_MAX_TEIS];			/* Max window size */
 	int v_s[Q921_MAX_TEIS];			/* Next N(S) for transmission */
 	int v_a[Q921_MAX_TEIS];			/* Last acknowledged frame */
 	int v_r[Q921_MAX_TEIS];			/* Next frame expected to be received */
 	int v_na[Q921_MAX_TEIS];			/* What we've told our peer we've acknowledged */
 	int solicitfbit[Q921_MAX_TEIS];	/* Have we sent an I or S frame with the F-bit set? */
 	int retrans[Q921_MAX_TEIS];		/* Retransmissions */

	int sentrej[Q921_MAX_TEIS];		/* Are we in reject state */
	
	/* Various timers */
 	int sabme_timer[Q921_MAX_TEIS];
 	int t203_timer[Q921_MAX_TEIS];
 	int t202_timer[Q921_MAX_TEIS];
 	int t201_timer[Q921_MAX_TEIS];
 	int t200_timer[Q921_MAX_TEIS];

	/* Used by scheduler */
	struct timeval tv;
	int schedev;
	pri_event ev;		/* Static event thingy */
	
	/* Q.921 Re-transmission queue */
	struct q921_frame *txqueue[Q921_MAX_TEIS];
	
	/* Q.931 calls */
	q931_call *calls;

	/* do we do overlap dialing */
	int overlapdial;

#ifdef LIBPRI_COUNTERS
	/* q921/q931 packet counters */
	unsigned int q921_txcount;
	unsigned int q921_rxcount;
	unsigned int q931_txcount;
	unsigned int q931_rxcount;
#endif
};

extern int pri_schedule_event(struct pri *pri, int ms, void (*function)(void *data), void *data);
extern int pri_schedule_event2(struct pri *pri, int ms, void (*function)(void *data, char data2), void *data, char data2);

extern pri_event *pri_schedule_run(struct pri *pri);

extern void pri_schedule_del(struct pri *pri, int ev);

extern pri_event *pri_mkerror(struct pri *pri, char *errstr);

extern void pri_message(char *fmt, ...);

extern void pri_error(char *fmt, ...);

#endif
